from pathlib import Path
from config import Config
from console import Console
from .localization import Localization
from .user import User
from .users_storage import UsersStorage


class Core:
    def __init__(self):
        self.config = Config()

        self.users_file = Path(self.config.database['file'])
        self.storage = UsersStorage(self.users_file)
        self.console = Console(self.config.console['prompt'])
        self.i18n = Localization(self.config.localization['dir'])

    def _add_user(self):
        while True:
            username = input(self.i18n.t("enter_username")).lower()
            if username in self.storage.get_index():
                print(self.i18n.t("username_exists"))
                continue
            break

        name = input(self.i18n.t("enter_name"))
        surname = input(self.i18n.t("enter_surname"))

        while True:
            age_input = input(self.i18n.t("enter_age"))
            if age_input.isdigit():
                age = int(age_input)
                break
            print(self.i18n.t("invalid_age"))

        while True:
            gender = input(self.i18n.t("enter_gender")).strip().lower()
            if gender in ("male", "female"):
                break
            print(self.i18n.t("invalid_gender"))

        # noinspection PyTypeChecker
        user = User(username=username, name=name, surname=surname, age=age, gender=gender)
        self.storage.add(user)
        print(self.i18n.t("user_added"))

    def _del_user(self):
        username = input(self.i18n.t("enter_username_delete")).lower()
        user = self.storage.find(username)
        if user:
            self.storage.delete(user)
            print(self.i18n.t("user_deleted"))
        else:
            print(self.i18n.t("user_not_found"))

    def _edit_user(self):
        username = input(self.i18n.t("enter_username_edit")).lower()
        user = self.storage.find(username)
        if not user:
            print(self.i18n.t("user_not_found"))
            return

        while True:
            new_username = input(self.i18n.t("enter_new_username", current=user.username)) or user.username
            if new_username != user.username and new_username in self.storage.get_index():
                print(self.i18n.t("username_exists"))
            else:
                break

        new_name = input(self.i18n.t("enter_new_name", current=user.name)) or user.name
        new_surname = input(self.i18n.t("enter_new_surname", current=user.surname)) or user.surname

        while True:
            new_age_input = input(self.i18n.t("enter_new_age", current=user.age)) or str(user.age)
            if new_age_input.isdigit():
                new_age = int(new_age_input)
                break
            print(self.i18n.t("invalid_age"))

        while True:
            new_gender = input(self.i18n.t("enter_new_gender", current=user.gender)) or user.gender
            if new_gender in ("male", "female"):
                break
            print(self.i18n.t("invalid_gender"))

        self.storage.delete(user)
        # noinspection PyTypeChecker
        updated_user = User(username=new_username, name=new_name, surname=new_surname, age=new_age, gender=new_gender)
        self.storage.add(updated_user)
        print(self.i18n.t("user_updated"))

    def _list_of_users(self):
        users = self.storage.list()
        if not users:
            print(self.i18n.t("no_users_found"))
            return

        headers = [
            self.i18n.t("header_username"),
            self.i18n.t("header_name"),
            self.i18n.t("header_surname"),
            self.i18n.t("header_age"),
            self.i18n.t("header_gender")
        ]
        attributes = ['username', 'name', 'surname', 'age', 'gender']

        # Вычисляем максимальную длину для каждого столбца
        max_lengths = []
        for i in range(len(headers)):
            max_len = max(len(headers[i]), max(len(str(getattr(user, attributes[i]))) for user in users))
            max_lengths.append(max_len)

        # Формируем строку заголовков
        header_line = " | ".join(headers[i].center(max_lengths[i]) for i in range(len(headers)))
        print(header_line)

        # Печатаем разделитель
        print("-" * len(header_line))

        # Выводим строки с данными пользователей
        for user in users:
            user_line = " | ".join(str(getattr(user, attributes[i])).center(max_lengths[i])
                                   for i in range(len(attributes)))
            print(user_line)

    def _change_language(self):
        available_languages = self.i18n.available_languages()
        print(self.i18n.t("available_languages"), ", ".join(available_languages))
        new_lang = input(self.i18n.t("choose_language"))
        self.i18n.set_language(new_lang)

        self.console.update_help_messages(self.i18n)

    def load(self):

        self.i18n.set_language(self.config.localization['default'])

        self.console.reg_command("help", self.i18n["cmd_help"], self.console.print_help)
        self.console.reg_command("add",  self.i18n["cmd_add"],  self._add_user)
        self.console.reg_command("del",  self.i18n["cmd_del"],  self._del_user)
        self.console.reg_command("edit", self.i18n["cmd_edit"], self._edit_user)
        self.console.reg_command("list", self.i18n["cmd_list"], self._list_of_users)
        self.console.reg_command("lang", self.i18n["cmd_lang"], self._change_language)
        self.console.reg_command("exit", self.i18n["cmd_exit"], self.console.stop)

        self.storage.load()

    def start(self):
        print("Welcome to UsersConsole v1.1!")
        self.console.start()
